<?php

// autoload_static.php @generated by Composer

namespace Composer\Autoload;

class ComposerStaticInitd06864f09a3f6a7ae214b7100b0b6536
{
    public static $files = array (
        'a2c78434f64e5f5ed402f42eee19c025' => __DIR__ . '/..' . '/ipl/stdlib/src/functions_include.php',
        '14a6a72370de4c08174524928551af59' => __DIR__ . '/..' . '/ipl/stdlib/src/polyfill-php71.php',
        '7b11c4dc42b3b3023073cb14e519683c' => __DIR__ . '/..' . '/ralouphie/getallheaders/src/getallheaders.php',
        'a0edc8309cc5e1d60e3047b5df6b7052' => __DIR__ . '/..' . '/guzzlehttp/psr7/src/functions_include.php',
    );

    public static $prefixLengthsPsr4 = array (
        'i' => 
        array (
            'ipl\\Web\\' => 8,
            'ipl\\Validator\\' => 14,
            'ipl\\Stdlib\\' => 11,
            'ipl\\Sql\\' => 8,
            'ipl\\Orm\\' => 8,
            'ipl\\Html\\' => 9,
        ),
        'P' => 
        array (
            'Psr\\Http\\Message\\' => 17,
        ),
        'G' => 
        array (
            'GuzzleHttp\\Psr7\\' => 16,
        ),
    );

    public static $prefixDirsPsr4 = array (
        'ipl\\Web\\' => 
        array (
            0 => __DIR__ . '/..' . '/ipl/web/src',
        ),
        'ipl\\Validator\\' => 
        array (
            0 => __DIR__ . '/..' . '/ipl/validator/src',
        ),
        'ipl\\Stdlib\\' => 
        array (
            0 => __DIR__ . '/..' . '/ipl/stdlib/src',
        ),
        'ipl\\Sql\\' => 
        array (
            0 => __DIR__ . '/..' . '/ipl/sql/src',
        ),
        'ipl\\Orm\\' => 
        array (
            0 => __DIR__ . '/..' . '/ipl/orm/src',
        ),
        'ipl\\Html\\' => 
        array (
            0 => __DIR__ . '/..' . '/ipl/html/src',
        ),
        'Psr\\Http\\Message\\' => 
        array (
            0 => __DIR__ . '/..' . '/psr/http-message/src',
        ),
        'GuzzleHttp\\Psr7\\' => 
        array (
            0 => __DIR__ . '/..' . '/guzzlehttp/psr7/src',
        ),
    );

    public static $prefixesPsr0 = array (
        'E' => 
        array (
            'Evenement' => 
            array (
                0 => __DIR__ . '/..' . '/evenement/evenement/src',
            ),
        ),
        'A' => 
        array (
            'AssetLoader' => 
            array (
                0 => __DIR__ . '/../..' . '/',
            ),
        ),
    );

    public static function getInitializer(ClassLoader $loader)
    {
        return \Closure::bind(function () use ($loader) {
            $loader->prefixLengthsPsr4 = ComposerStaticInitd06864f09a3f6a7ae214b7100b0b6536::$prefixLengthsPsr4;
            $loader->prefixDirsPsr4 = ComposerStaticInitd06864f09a3f6a7ae214b7100b0b6536::$prefixDirsPsr4;
            $loader->prefixesPsr0 = ComposerStaticInitd06864f09a3f6a7ae214b7100b0b6536::$prefixesPsr0;

        }, null, ClassLoader::class);
    }
}
